@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/osrs/libtiff/tools/fax2ps.c,v 1.2 1999/08/17 03:07:43 warmerda Exp $" */
+/* $Header: /cvsroot/osrs/libtiff/tools/fax2ps.c,v 1.5 2001/03/29 01:45:43 warmerda Exp $" */
 
 /*
  * Copyright (c) 1991-1997 Sam Leffler
@@ -62,7 +62,7 @@ int	maxline = 512;		/* max output line of PostScript */
  * March 13-15, 1995.
  */
 static void
-printruns(unsigned char* buf, uint16* runs, uint16* erun, uint32 lastx)
+printruns(unsigned char* buf, uint32* runs, uint32* erun, uint32 lastx)
 {
     static struct {
 	char white, black;
@@ -155,7 +155,7 @@ printTIF(TIFF* tif, int pageNumber)
 {
     uint32 w, h;
     uint16 unit;
-    float xres, yres;
+    float xres, yres, scale = 1.0;
     tstrip_t s, ns;
 
     TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
@@ -178,21 +178,13 @@ printTIF(TIFF* tif, int pageNumber)
 
     printf("%%%%Page: \"%d\" %d\n", pageNumber, pageNumber);
     printf("/$pageTop save def gsave\n");
-    if (scaleToPage) {
-	float yscale = pageHeight / (h/yres);
-	float xscale = pageWidth / (w/xres);
-	printf("%d %d translate\n",
-               (int) (((basePageWidth - pageWidth) * points) * half),
-               (int)((yscale*(h/yres)*points) +
-               (basePageHeight - pageHeight) * points * half)  );
-	printf("%g %g scale\n", (72.*xscale)/xres, -(72.*yscale)/yres);
-    } else {
-	printf("%d %d translate\n",
-               (int) ((basePageWidth - pageWidth) * points * half),
-               (int)((72.*h/yres) +
-               (basePageHeight - pageHeight) * points * half) );
-	printf("%g %g scale\n", 72./xres, -72./yres);
-    }
+    if (scaleToPage)
+        scale = pageHeight / (h/yres) < pageWidth / (w/xres) ?
+            pageHeight / (h/yres) : pageWidth / (w/xres);
+    printf("%g %g translate\n",
+           points * (pageWidth - scale*w/xres) * half,
+           points * (scale*h/yres + (pageHeight - scale*h/yres) * half));
+    printf("%g %g scale\n", points/xres*scale, -points/yres*scale);
     printf("0 setgray\n");
     TIFFSetField(tif, TIFFTAG_FAXFILLFUNC, printruns);
     ns = TIFFNumberOfStrips(tif);
@@ -356,7 +348,7 @@ main(int argc, char** argv)
     printf("%%%%CreationDate: %s", ctime(&t));
     printf("%%%%Origin: 0 0\n");
     printf("%%%%BoundingBox: 0 0 %u %u\n",
-	(int)(pageHeight*72), (int)(pageWidth*72));	/* XXX */
+	(int)(pageWidth*72), (int)(pageHeight*72));	/* XXX */
     printf("%%%%Pages: (atend)\n");
     printf("%%%%EndComments\n");
     printf("%%%%BeginProlog\n");
