@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/osrs/libtiff/tools/tiffmedian.c,v 1.2 1999/12/21 17:03:03 mwelles Exp $ */
+/* $Header: /cvsroot/osrs/libtiff/tools/tiffmedian.c,v 1.3 1999/12/27 17:35:01 mwelles Exp $ */
 
 /*
  * Apply median cut on an image.
@@ -7,7 +7,7 @@
  *     -C n		- set colortable size.  Default is 256.
  *     -f		- use Floyd-Steinberg dithering.
  *     -c lzw		- compress output with LZW 
- *                        (no longer supported due to unisys patent enforcement) 
+ *                        (no longer supported by default due to unisys patent enforcement) 
  *     -c none		- use no compression on output
  *     -c packbits	- use packbits compression on output
  *     -r n		- create output with n rows/strip of data
@@ -314,7 +314,7 @@ char* stuff[] = {
 " -C #		create a colormap with # entries",
 " -f		use Floyd-Steinberg dithering",
 " -c lzw[:opts]	compress output with Lempel-Ziv & Welch encoding",
-"               (no longer supported due to Unisys patent enforcement)", 
+"               (no longer supported by default due to Unisys patent enforcement)", 
 " -c zip[:opts]	compress output with deflate encoding",
 " -c packbits	compress output with packbits encoding",
 " -c none	use no compression algorithm on output",
